#!/usr/bin/env python
# vim: set fileencoding=latin-1 :
#bhrss.py

"""CGI script to provide a RSS feed of currently blocked and watched IP
addresses.

------------------- NOT READY FOR USE YET [Feb 2007]
------------------- EXPECTED TO BE AVAILABLE SOON

====
bhrss.py Script License
This work is hereby released into the Public Domain.
To view a copy of the public domain dedication, visit
http://creativecommons.org/licenses/publicdomain/ or send a letter to
Creative Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.

Author: Avinash Chopde <avinash@acm.org>
Created: February 2007
http://www.aczoom.com/cms/blockhosts/

"""

# script metadata, also used by setup.py
SCRIPT_ID="bhrss"
VERSION="1.0.0"
VERSION_DATE="February 2007"
AUTHOR="Avinash Chopde"
AUTHOR_EMAIL="avinash@acm.org"
URL="http://www.aczoom.com/cms/blockhosts/"
LICENSE="http://creativecommons.org/licenses/publicdomain/"
DESCRIPTION="Send email regarding BlockHosts blocked IP addresses"
LONG_DESCRIPTION=DESCRIPTION

# -------------------------------------------------------------
import syslog
if __name__ == '__main__':
    syslog.openlog(SCRIPT_ID)

#-------------------------------------
import re
import os, os.path, sys, traceback
import copy
import string
import time
import errno
import smtplib
import ConfigParser
# from RSS import ns, CollectionChannel

import blockhosts
from blockhosts import die, debug, warning, info, error, print_always, sort_by_value

try:
    import datetime
    from optparse import OptionParser, OptionGroup
except ImportError, e:
    print >> sys.stderr, "** Missing modules: datetime or optparse\nWill not work with earlier python versions - 2.3 or later needed.\n", e
    raise

# --------------------------------------
# defaults for parameters follow this order:
# 1 -> use the value provided as an argument in argv[] to this script
# 2 -> if not, then use the value defined in CONFIG_FILE
# 3 -> if not, then use the value hard-coded in this script - HC_OPTIONS

# Defaults, hard-coded options, these values are used last if no args
# and no config file provided
HC_OPTIONS = {
    "HOSTS_BLOCKFILE": blockhosts.HC_OPTIONS["HOSTS_BLOCKFILE"],
}

#######################################################################
def main():
    """Collects args, open block-file, deliver list of blocked addresses"""

    args = sys.argv[1:]

    oparser = OptionParser(version=VERSION, description=DESCRIPTION)

    config_options = copy.copy(HC_OPTIONS)
    blockhosts.get_config(config_options)
    #DEFAULTS
    oparser.set_defaults(verbose=1,
        echo="",
        blockfile=config_options["HOSTS_BLOCKFILE"],
        )

    defaults = oparser.get_default_values()

    oparser.add_option("-q", "--quiet",
        action="store_const", const=0, dest="verbose",
        help="Be as quiet as possible - only print out error messages")

    oparser.add_option("-v", "--verbose",
        action="store_const", const=2, dest="verbose",
        help="Be verbose - print errors, warnings, and info messages")

    oparser.add_option("-g", "--debug",
        action="store_const", const=3, dest="verbose",
        help="Be chatty - print out debug level messages also")

    oparser.add_option("--echo", type="string", metavar="TAG",
        help="Prints TAG on stderr and syslog, may be used to identify a run of %s (%s)" % (SCRIPT_ID, defaults.echo))

    oconfig = OptionGroup(oparser, "Configuration Options",
        """Hard-coded defaults can be overridden by the values in the
config file - if it exists at %s, and those values can be overriden by
using the command-line options.  The current values are shown in () below.
This program uses a section in the block file (%s) to find blockhosts data.
This section is demarcated by two identical marker lines, with this text
- without the quotes: "%s".
Also see the "INSTALL" file in the blockhosts source package for a
detailed example of the block file.
""" % (blockhosts.CONFIG_FILE, defaults.blockfile, blockhosts.HOSTS_MARKER_LINE))

    oconfig.add_option("--blockfile", type="string", metavar="FILE",
        help="Name of hosts-block-file to read (%s)" % defaults.blockfile)

    oparser.add_option_group(oconfig)

    (options, remaining_args) = oparser.parse_args()

    # set same logging/message level in blockhosts as in this script
    blockhosts.MESSAGE_LEVEL = options.verbose

    info("%s %s started: %s" % (SCRIPT_ID, VERSION, blockhosts.NOW_DATETIME.strftime("%Y-%m-%d %H:%M:%S")))
    if options.echo:
        # force printing echo tag to screen, and to syslog
        syslog.syslog(syslog.LOG_INFO, "echo tag: %s" % options.echo)
        print_always(" ... echo tag: %s" % options.echo)

    debug("Got options:", options)

    if remaining_args:
        warning("ignoring positional arguments - there should be none!", remaining_args)

    debug("Debug mode enabled.")

    #-----------------------------------------------------------------
    # all options/args collected, construct output message

    # load all currently blocked hosts
    dh = blockhosts.BlockHosts(options.blockfile)
    try:
        dh.load_hosts_blockfile()
    except (blockhosts.MissingMarkerError, blockhosts.SecondMarkerError):
        die("Failed to load blockfile - block-file marker error\n Expected two marker lines in the file,\n somewhere in the middle of the file:\n%s\n%s\n" % (HOSTS_MARKER_LINE, HOSTS_MARKER_LINE))
    except:
        traceback.print_exc()
        die("Failed to load blockfile")

    watched_hosts = dh.get_initial_watched_hosts()
    blocked_hosts = dh.get_initial_deny_hosts()

    # get list of hosts, sorted by datetime, in descending order
    # hosts = sort_by_value(watched_hosts, reverse = True)
    # hosts = watched_hosts.keys() # unsorted, unspecified order

    lines = ["--- list of all blocked hosts--\n",]
    subject = SCRIPT_ID + ": currently blocked addresses."

    # always include all blocked hosts in email body
    for host in blocked_hosts:
        try:
            data = watched_hosts[host]
        except KeyError:
            error("%s: found blocked IP (%s), but not in watched list (blockhosts.py error?)." % (SCRIPT_ID, host))
            line = " %15s, <no data - possible error in blockhosts.py?>" % host
        else:
            line = " %15s, count %2d, added %s" % (host, data.count, data.datetime.strftime("%Y-%m-%d-%H-%M"))

        lines.append(line)
    lines.append("---")

    if len(blocked_hosts) > 0:
        # only send text if there are blocked hosts
        print "\n".join(lines)
    else:
        info("bhrss: no blocked hosts to send.")

    die("bhrss: this script is not yet ready for release")

    debug("bhrss: all done.")

#######################################################################
if __name__ == '__main__':
    sys.exit(main())
